| Phi2rho-package | R Documentation |
Computes the Owen's T function or the bivariate normal integral.
The DESCRIPTION file:
Package: Phi2rho
Type: Package
Title: Owen's T Function and Bivariate Normal Integral
Version: 1.0.2
Date: 2026-06-27
Authors@R: person("Janez","Komelj", role = c("aut","cre"),
email = "jkomelj@siol.net")
Depends: R (>= 3.5.0), stats
Imports: Rmpfr
Description: Computes 'Owen's T' function or the bivariate normal integral
using one of the following methods: the modified Euler's arctangent
series, tetrachoric series, or Vasicek's series. For the methodology,
see Komelj, J. (2023) <doi:10.4236/ajcm.2023.134026>, or the revised
postprint version with corrected contents and improved typography at
<doi:10.48550/arXiv.2312.00011>. Also see the alternative approach by
Vasicek, O. A. (1998) <doi:10.21314/JCF.1998.015>.
License: GPL-2 | GPL-3
Janez Komelj
Maintainer: Janez Komelj <jkomelj@siol.net>
Komelj, J. (2023): The Bivariate Normal Integral via Owen's T Function as a Modified Euler's Arctangent Series, American Journal of Computational Mathematics, 13, 4, 476–504, \Sexpr[results=rd]{tools:::Rd_expr_doi("10.4236/ajcm.2023.134026")}, or the revised postprint version with corrected contents and improved typography at \Sexpr[results=rd]{tools:::Rd_expr_doi("10.48550/arXiv.2312.00011")}.
Owen, D. B. (1956): Tables for Computing Bivariate Normal Probabilities, The Annals of Mathematical Statistics, 27, 4, 1075–1090, \Sexpr[results=rd]{tools:::Rd_expr_doi("10.1214/aoms/1177728074")}.
Owen, D. B. (1980): A table of normal integrals, Communications in Statistics – Simulation and Computation, 9, 4, 389–419, \Sexpr[results=rd]{tools:::Rd_expr_doi("10.1080/03610918008812164")}.
Vasicek, O. A. (1998): A series expansion for the bivariate normal integral, The Journal of Computational Finance, 1, 4, 5–10, \Sexpr[results=rd]{tools:::Rd_expr_doi("10.21314/JCF.1998.015")}.
OwenT(2, 0.5)
OwenT(2, 0.5, fun = "mOwenT") # modified arctangent series (default)
OwenT(2, 0.5, fun = "tOwenT") # tetrachoric series
OwenT(2, 0.5, fun = "vOwenT") # Vasicek's series
rho <- 0.6
a <- rho/sqrt(1 - rho^2)
OwenT(0.3, a)
OwenT(0.3, a, fun = "tOwenT")
OwenT(c(-1, 0.5, 4), a, fun = "vOwenT")
OwenT(2, c(-1, -0.5, 0, 0.5, 1), fun = "vOwenT")
Phi2xy(2, 1.3, 0.5)
Phi2xy(-2, 0.5, -0.3, fun = "tOwenT")
Phi2xy(c(1, 2, -1.5), c(-1, 1, 2.3), 0.5, fun = "vOwenT")
Phi2xy(1, 2, c(-1, -0.5, 0, 0.5, 1))
Phi2xy(c(1, 2), c(-1,3), c(-0.5, 0.8))
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.